/*  File: analyser-ZQ.h
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 14 September 2016
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __analyzer_zQ_h__
#define __analyzer_zQ_h__

/* Deductor headers */
#include <user.h>

using namespace std;
using namespace duct;
using namespace duct::distpoint;

/*   Narrowing the basic_user template   */
using DistVoid = distbook<double, void>;
using DistHist1d = distbook<double, hist1d>;
using DistRatio4 = distbook<std::array<double,4>, hist1d,
                   sample_ratio_traits<std::array<double,4>>>;

/*
 *   This is the declaration of our analyzer class.
 */
class UserZQ : public basic_user<hhc, DistVoid, DistHist1d, DistRatio4>
{
  /*   useful aliases  */
  using Hist0D = Phys<DistVoid>;
  using Hist1D = Phys<DistHist1d>;
  using Hist1DR4 = Phys<DistRatio4>;
  
public:
  /*   constructors   */
  UserZQ() = default;
  UserZQ(const char *n) {this->name(n);}
  
  int count;
  
  /*   Initializing routine, called once
   *   at the beginning of the calculation.
   */
  void initfunc();
  
  /*   The analyzer routine. It analyses the events and fills the histograms. */
  void userfunc(const shower_history<hhc>&);
};

#endif


