/*  File: analyser-ZpT.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 15 September 2016.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   local headers  */
#include "analyzer-ZpT.h"

void UserZpT::initfunc()
{
  /*   Creating a histogram to calculate the pT distribution   */
  unsigned nbins = 100;
  double pTmin = 0.0_GeV, pTmax = 100.0_GeV;
  auto bins = spacing<hist1d>::linear(nbins, pTmin, pTmax);
  bool enable_norm = true;
  
  Hist1D::phys(1, "$Z/\\gamma$ $p_T$ distribution, Full", bins, enable_norm);
  Hist1D::phys(2, "$Z/\\gamma$ $p_T$ distribution, No Delta", bins, enable_norm);
  Hist1D::phys(3, "$Z/\\gamma$ $p_T$ distribution, ThresholdMain", bins, enable_norm);
  Hist1D::phys(4, "$Z/\\gamma$ $p_T$ distribution, Std", bins, enable_norm);
  Hist1DR4::phys(5, "$Z/\\gamma$ $p_T$ distribution - ratios", bins);
}


void UserZpT::userfunc(const shower_history<hhc>& history)
{
  /*   Conversion factor from 1/GeV^2 to nb    */
  constexpr double tonb = 389379.338;

  /*   the shower stage after the shower evolution   */
  for(auto it = history.cbegin(); it != history.cend(); ++it)
  {
    /*   momentum of the e+e- pair   */
    auto& p = it->state;
    auto pee = p[-2].momentum + p[-3].momentum;
    
    /*   Calculate the weights   */

    auto& weights = it->weights;
    // The weight for a probability conserving shower
    // including factor to use NLO pdfs at hard scale.
    double wStd = weights.unitary.main;
           wStd*= weights.unitary.pdf_ratio*color_weight(*it)*tonb;
    // The main threshold factor, omitting P^{reg} term and the Delta term.
    double wThresholdMain = wStd*weights.threshold.main;
    // Now include the P^{reg} term.
    double wNoDelta = wThresholdMain*weights.threshold.reg;
    // Now include the Delta term also, giving the full result.
    double wFull = wNoDelta*weights.threshold.delta;
    
    std::array<double,4> myweights4;
    myweights4[0] = wFull;
    myweights4[1] = wNoDelta;
    myweights4[2] = wThresholdMain;
    myweights4[3] = wStd;

    /*   Fill the histograms   */
    if(pee.mag() > 2000.0 && pee.mag() < 2100.0 && pee.perp() < 100.0) {
      Hist1D::normfill(1, wFull);
      Hist1D::physfill(1, wFull, dirac{}, pee.perp());
      Hist1D::normfill(2, wNoDelta);
      Hist1D::physfill(2, wNoDelta, dirac{}, pee.perp());
      Hist1D::normfill(3, wThresholdMain);
      Hist1D::physfill(3, wThresholdMain, dirac{}, pee.perp());
      Hist1D::normfill(4, wStd);
      Hist1D::physfill(4, wStd, dirac{}, pee.perp());
      Hist1DR4::physfill(5, myweights4, dirac{}, pee.perp());
    }
  }
}



