/*  File: mod-dy.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 15 September 2016.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


/*   Deductor headers  */
#include <deductor.h>
#include <proc-dyjets.h>

/*   Analyser headers  */
#include "analyzer-ZQ.h"

/*   Deductor and Standard namespaces   */
using namespace duct;
using namespace std;
using namespace std::literals::chrono_literals;

/* This function defines what Deductor should do. */

template<class _Ordering>
void my_DY_module(const deductor_input& info)
{
  /* deductor object */
  deductor<hhc> kernel{qcd::ct14n};
  
  /* We create a function object that can create the desired hard process. */
  auto hard = [=](const model_ref& mdlr) -> dyjets
  {
    double Ecm = 13.0_TeV;
    double scalefactor = 1.0;
    double mL = 200.0_GeV, mH = 7.2_TeV;
    int proton = 2212, electron = 11;
    
    return dyjets(mdlr, Ecm, mL, mH, scalefactor, proton, proton, {electron});
  };
  
  /*  Create the shower process, start the calculation and return the process ID.*/
  shower_engine_opts opts;
  opts.max_color_suppression = 0u;
  opts.allow_color_branching = false; // The default is true.
  opts.threshold.psi_min = 0.01;      // The default is 0.01.

  kernel("Drell-Yan", opts, hard, new UserZQ{"ZQ"});
  cout << "Calculation started." << endl;
}

/*   Always export your module functions. */
export_my_module drell_yan_module = {
  {"DY-13TeV",  my_DY_module<ordering::lambda>},
};

